"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = __importDefault(require("lodash"));
const userPermissionDependencies = [
    {
        name: "ViewAllData",
        permissionsRequired: ["ViewPlatformEvents", "ViewDataLeakageEvents"],
        hasAccessOnData: true
    },
    {
        name: "QueryAllFiles",
        hasAccessOnData: true
    },
    {
        name: "InstallPackaging",
        permissionsRequired: ["ViewDataLeakageEvents", "EditPublicReports"]
    },
    {
        name: "CanUseNewDashboardBuilder",
        permissionsRequired: ["ManageDashboards"]
    },
    {
        name: "ScheduleReports",
        permissionsRequired: ["EditReports", "RunReports"]
    },
    {
        name: "EditReports",
        permissionsRequired: ["RunReports"]
    },
    {
        name: "ModifyAllData",
        permissionsRequired: ["EditPublicReports", "ManageDashboards"]
    },
    {
        name: "BulkMacrosAllowed",
        objectsAccessRequired: [
            {
                object: "Macro",
                allowCreate: "false",
                allowDelete: "false",
                allowEdit: "false",
                allowRead: "true",
                modifyAllRecords: "false",
                viewAllRecords: "false"
            }
        ]
    },
    {
        name: "ManageSolutions",
        objectsAccessRequired: [
            {
                object: "Solution",
                allowCreate: "true",
                allowDelete: "true",
                allowEdit: "true",
                allowRead: "true",
                modifyAllRecords: "false",
                viewAllRecords: "false"
            }
        ]
    },
    {
        name: "ManageCssUsers",
        objectsAccessRequired: [
            {
                object: "Contact",
                allowCreate: "true",
                allowDelete: "false",
                allowEdit: "true",
                allowRead: "true",
                modifyAllRecords: "false",
                viewAllRecords: "false"
            }
        ]
    },
    {
        name: "TransferAnyCase",
        objectsAccessRequired: [
            {
                object: "Case",
                allowCreate: "true",
                allowDelete: "false",
                allowEdit: "false",
                allowRead: "true",
                modifyAllRecords: "false",
                viewAllRecords: "false"
            }
        ]
    }
];
class UserPermissionBuilder {
    static addPermissionDependencies(profileOrPermissionSet) {
        let objectAccessRequired = [];
        for (let i = 0; i < userPermissionDependencies.length; i++) {
            let dependedPermission = userPermissionDependencies[i];
            if (profileOrPermissionSet.userPermissions != null &&
                profileOrPermissionSet.userPermissions.length > 0) {
                for (let j = 0; j < profileOrPermissionSet.userPermissions.length; j++) {
                    let permission = profileOrPermissionSet.userPermissions[j];
                    if (permission.name == dependedPermission.name) {
                        objectAccessRequired.push(...dependedPermission.objectsAccessRequired);
                    }
                }
            }
        }
        if (objectAccessRequired.length > 0) {
            UserPermissionBuilder.addRequiredObjectAccess(profileOrPermissionSet, UserPermissionBuilder.mergeObjectAccess(objectAccessRequired));
        }
    }
    static mergeObjectAccess(objectAccessRequired) {
        var objectMapping = {};
        for (var i = 0; i < objectAccessRequired.length; i++) {
            var objectAccess = objectAccessRequired[i];
            if (objectMapping[objectAccess.object] != undefined) {
                //console.log('Adding access');
                UserPermissionBuilder.addAccess(objectMapping[objectAccess.object], objectAccess);
            }
            else {
                //console.log('object access does not exists ');
                objectMapping[objectAccess.object] = objectAccess;
            }
        }
        return Object.values(objectMapping);
    }
    static addAccess(objectAccess1, ObjectAccess2) {
        objectAccess1.allowCreate =
            objectAccess1.allowCreate.toString() === "true"
                ? true
                : ObjectAccess2.allowCreate;
        objectAccess1.allowDelete =
            objectAccess1.allowDelete.toString() === "true"
                ? true
                : ObjectAccess2.allowDelete;
        objectAccess1.allowEdit =
            objectAccess1.allowEdit.toString() === "true"
                ? true
                : ObjectAccess2.allowEdit;
        objectAccess1.allowRead =
            objectAccess1.allowRead.toString() === "true"
                ? true
                : ObjectAccess2.allowRead;
        objectAccess1.modifyAllRecords =
            objectAccess1.modifyAllRecords.toString() === true
                ? true
                : ObjectAccess2.modifyAllRecords;
        objectAccess1.viewAllRecords =
            objectAccess1.viewAllRecords.toString() === "true"
                ? true
                : ObjectAccess2.viewAllRecords;
    }
    static addRequiredObjectAccess(profileOrPermissionSet, objectAccessRequired) {
        if (profileOrPermissionSet.objectPermissions == null ||
            profileOrPermissionSet.objectPermissions == undefined ||
            !Array.isArray(profileOrPermissionSet.objectPermissions)) {
            profileOrPermissionSet.objectPermissions = objectAccessRequired;
        }
        else {
            let objectAccesses = objectAccessRequired.filter(objectAccess => {
                let exist = false;
                for (let i = 0; i < profileOrPermissionSet.objectPermissions.length; i++) {
                    let profileObjectAccess = profileOrPermissionSet.objectPermissions[i];
                    exist = profileObjectAccess.object == objectAccess.object;
                    if (exist) {
                        UserPermissionBuilder.addAccess(profileObjectAccess, objectAccess);
                        break;
                    }
                }
                return !exist;
            });
            if (objectAccesses.length > 0) {
                profileOrPermissionSet.objectPermissions.push(...objectAccesses);
            }
        }
    }
    static async describeSObject(conn, sObjectName) {
        var toReturn = new Promise((resolve, reject) => {
            conn
                .sobject(sObjectName)
                .describe(function (err, meta) {
                if (err) {
                    console.error(err);
                    reject(err);
                }
                resolve(meta);
            });
        });
        return toReturn;
    }
    static async getSupportedPermissions(conn) {
        if (lodash_1.default.isNil(UserPermissionBuilder.supportedPermissions)) {
            let describeResult = await UserPermissionBuilder.describeSObject(conn, "PermissionSet");
            UserPermissionBuilder.supportedPermissions = [];
            describeResult.fields.forEach(field => {
                let fieldName = field["name"];
                if (fieldName.startsWith("Permissions")) {
                    UserPermissionBuilder.supportedPermissions.push(fieldName.replace("Permissions", "").trim());
                }
            });
        }
        return UserPermissionBuilder.supportedPermissions;
    }
    static async isSupportedPermission(permission) {
        let found = false;
        if (!lodash_1.default.isNil(UserPermissionBuilder.supportedPermissions)) {
            found = UserPermissionBuilder.supportedPermissions.includes(permission);
        }
        return found;
    }
    static handlePermissionDependency(profileOrPermissionSet, supportedPermissions) {
        userPermissionDependencies.forEach(userPermission => {
            let hasPermission = UserPermissionBuilder.hasPermission(profileOrPermissionSet, userPermission.name);
            if (hasPermission &&
                userPermission.hasAccessOnData &&
                profileOrPermissionSet.objectPermissions !== undefined &&
                profileOrPermissionSet.objectPermissions.length > 0) {
                for (var i = 0; i < profileOrPermissionSet.objectPermissions.length; i++) {
                    profileOrPermissionSet.objectPermissions[i].allowRead = true;
                    profileOrPermissionSet.objectPermissions[i].viewAllRecords = true;
                }
            }
            if (hasPermission &&
                userPermission.permissionsRequired !== undefined &&
                userPermission.permissionsRequired.length > 0) {
                for (let i = 0; i < userPermission.permissionsRequired.length; i++) {
                    UserPermissionBuilder.enablePermission(profileOrPermissionSet, userPermission.permissionsRequired[i], supportedPermissions);
                }
            }
        });
    }
    static enablePermission(profileObj, permissionName, supportedPermission) {
        let found = false;
        if (profileObj.userPermissions !== undefined &&
            profileObj.userPermissions.length > 0) {
            for (var i = 0; i < profileObj.userPermissions.length; i++) {
                let element = profileObj.userPermissions[i];
                if (element.name === permissionName) {
                    element.enabled = true;
                    found = true;
                    break;
                }
            }
        }
        if (!found) {
            if (lodash_1.default.isNil(profileObj.userPermissions)) {
                profileObj.userPermissions = [];
            }
            if (supportedPermission.includes(permissionName)) {
                let permission = {
                    name: permissionName,
                    enabled: true
                };
                profileObj.userPermissions.push(permission);
            }
        }
    }
    static hasPermission(profileOrPermissionSet, permissionName) {
        let found = false;
        if (!lodash_1.default.isNil(profileOrPermissionSet.userPermissions)) {
            for (var i = 0; i < profileOrPermissionSet.userPermissions.length; i++) {
                let element = profileOrPermissionSet.userPermissions[i];
                if (element.name === permissionName) {
                    found = element.enabled;
                    break;
                }
            }
        }
        return found;
    }
}
exports.default = UserPermissionBuilder;
//# sourceMappingURL=userPermissionBuilder.js.map